from core.modules.vistrails_module import Module, ModuleError
from Scientific.IO import NetCDF
import numpy
import vtk

from SelfeBase import SelfeData

class Utility(object):
    namespace = 'Utilities'

class SelfeArrayToVTKScalars(Utility, Module):
    name = 'GridPak Array To VTK Scalars'

    def compute(self):
        s_ar = self.getInputFromPort("Scalars").get_array()
        ds = self.getInputFromPort("VTK Data")
        name = self.forceGetInputFromPort("Array Name")
        if name == None:
            name = "scalars"
            
        v_ar = vtk.vtkFloatArray()
        v_ar.SetNumberOfValues(s_ar.size)
        i = 0
        for z in range(s_ar.shape[0]):
            for x in range(s_ar.shape[2]):
                for y in range(s_ar.shape[1]):
                    v = s_ar[z,y,x]
                    v_ar.SetValue(i, v)
                    i += 1
                    
        v_ar.SetName(name)
        ds.vtkInstance.GetPointData().AddArray(v_ar)
        set = self.forceGetInputFromPort("Activate Scalars")
        if set == None:
            set = True
            
        if set:
            ds.vtkInstance.GetPointData().SetScalars(v_ar)

        self.setResult("Output Mesh", ds)

    @classmethod
    def register(cls,reg,basic):
        reg.add_module(cls, name=cls.name, namespace=cls.namespace)
        reg.add_input_port(cls, "VTK Data", (reg.get_module_by_name('edu.utah.sci.vistrails.vtk','vtkStructuredGrid'), 'Input Mesh'))
        reg.add_input_port(cls, "Activate Scalars", (basic.Boolean, 'Set active scalars'),True)
        reg.add_input_port(cls, "Array Name", (basic.String, 'Array name'))
        reg.add_input_port(cls, "Scalars", (reg.get_module_by_name('edu.utah.sci.vistrails.numpyscipy', 'Numpy Array', 'numpy|array'), 'Output Array'))
        reg.add_output_port(cls, "Output Mesh", (reg.get_module_by_name('edu.utah.sci.vistrails.vtk','vtkStructuredGrid'), 'Output Mesh'))

class SelfeArrayToVTKVectors(Utility, Module):
    name = 'GridPak Arrays to VTK Vectors'

    def compute(self):
        v0 = self.forceGetInputFromPort("Array0")
        v1 = self.forceGetInputFromPort("Array1")
        v2 = self.forceGetInputFromPort("Array2")
        ds = self.getInputFromPort("VTK Data")
        name = self.forceGetInputFromPort("Array Name")

        vsize = 0
        if v0:
            vsize += 1
            v0 = v0.get_array()
        if v1:
            vsize += 1
            v1 = v1.get_array()
        if v2:
            vsize += 1
            v2 = v2.get_array()
            
        if name == None:
            name = "vectors"
            
        v_ar = vtk.vtkFloatArray()
        v_ar.SetNumberOfComponents(3)
        i = 0
        for z in range(v0.shape[0]):
            for y in range(v0.shape[2]):
                for x in range(v0.shape[1]):
                    u = v0[z,x,y]
                    v = v1[z,x,y]
                    w = v2[z,x,y]
                    v_ar.InsertNextTuple3(u, v, w)
                    i += 1

        v_ar.SetName(name)
        ds.vtkInstance.GetPointData().AddArray(v_ar)
        ds.vtkInstance.GetPointData().SetVectors(v_ar)
        self.setResult("Output Mesh", ds)
        print ds.vtkInstance
        
    @classmethod
    def register(cls,reg,basic):
        reg.add_module(cls, name=cls.name, namespace=cls.namespace)
        reg.add_input_port(cls, "VTK Data", (reg.get_module_by_name('edu.utah.sci.vistrails.vtk','vtkStructuredGrid'), 'Input Mesh'))
        reg.add_input_port(cls, "Array Name", (basic.String, 'Array name'))
        reg.add_input_port(cls, "Array0", (reg.get_module_by_name('edu.utah.sci.vistrails.numpyscipy', 'Numpy Array', 'numpy|array'), 'Scalar Array'))
        reg.add_input_port(cls, "Array1", (reg.get_module_by_name('edu.utah.sci.vistrails.numpyscipy', 'Numpy Array', 'numpy|array'), 'Scalar Array'))
        reg.add_input_port(cls, "Array2", (reg.get_module_by_name('edu.utah.sci.vistrails.numpyscipy', 'Numpy Array', 'numpy|array'), 'Scalar Array'))
        reg.add_output_port(cls, "Output Mesh", (reg.get_module_by_name('edu.utah.sci.vistrails.vtk','vtkStructuredGrid'), 'Output Mesh'))
        
